@@ -1,5 +1,5 @@
 /* intrin.c -- Recognize references to intrinsics
-   Copyright (C) 1995-1997 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998 Free Software Foundation, Inc.
    Contributed by James Craig Burley (burley@gnu.ai.mit.edu).
 
 This file is part of GNU Fortran.
@@ -27,6 +27,7 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "expr.h"
 #include "info.h"
 #include "src.h"
+#include "symbol.h"
 #include "target.h"
 #include "top.h"
 
@@ -58,9 +59,10 @@ struct _ffeintrin_spec_
 struct _ffeintrin_imp_
   {
     char *name;			/* Name of implementation. */
-    ffeintrinImp cg_imp;	/* Unique code-generation code. */
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
-    ffecomGfrt gfrt;		/* gfrt index in library. */
+    ffecomGfrt gfrt_direct;	/* library routine, direct-callable form. */
+    ffecomGfrt gfrt_f2c;	/* library routine, f2c-callable form. */
+    ffecomGfrt gfrt_gnu;	/* library routine, gnu-callable form. */
 #endif	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
     char *control;
   };
@@ -82,14 +84,12 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   { UPPER, LOWER, MIXED, FFEINTRIN_ ## GEN, FFEINTRIN_ ## SPEC },
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2)
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP)
-#define DEFIMP(CODE,NAME,GFRT,CONTROL)
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP)
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL)
 #include "intrin.def"
 #undef DEFNAME
 #undef DEFGEN
 #undef DEFSPEC
 #undef DEFIMP
-#undef DEFIMQ
 };
 
 static struct _ffeintrin_gen_ ffeintrin_gens_[]
@@ -99,14 +99,12 @@ static struct _ffeintrin_gen_ ffeintrin_gens_[]
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2) \
   { NAME, { SPEC1, SPEC2, }, },
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP)
-#define DEFIMP(CODE,NAME,GFRT,CONTROL)
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP)
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL)
 #include "intrin.def"
 #undef DEFNAME
 #undef DEFGEN
 #undef DEFSPEC
 #undef DEFIMP
-#undef DEFIMQ
 };
 
 static struct _ffeintrin_imp_ ffeintrin_imps_[]
@@ -116,15 +114,12 @@ static struct _ffeintrin_imp_ ffeintrin_imps_[]
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2)
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP)
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
-#define DEFIMP(CODE,NAME,GFRT,CONTROL) \
-      { NAME, FFEINTRIN_imp ## CODE, FFECOM_gfrt ## GFRT, CONTROL },
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP) \
-      { NAME, FFEINTRIN_imp ## CGIMP, FFECOM_gfrt ## GFRT, CONTROL },
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL) \
+      { NAME, FFECOM_gfrt ## GFRTDIRECT, FFECOM_gfrt ## GFRTF2C, \
+	FFECOM_gfrt ## GFRTGNU, CONTROL },
 #elif FFECOM_targetCURRENT == FFECOM_targetFFE
-#define DEFIMP(CODE,NAME,GFRT,CONTROL) \
-      { NAME, CODE, CONTROL },
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP) \
-      { NAME, CGIMP, CONTROL },
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL) \
+      { NAME, CONTROL },
 #else
 #error
 #endif
@@ -133,7 +128,6 @@ static struct _ffeintrin_imp_ ffeintrin_imps_[]
 #undef DEFGEN
 #undef DEFSPEC
 #undef DEFIMP
-#undef DEFIMQ
 };
 
 static struct _ffeintrin_spec_ ffeintrin_specs_[]
@@ -143,13 +137,11 @@ static struct _ffeintrin_spec_ ffeintrin_specs_[]
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2)
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP) \
   { NAME, CALLABLE, FAMILY, IMP, },
-#define DEFIMP(CODE,NAME,GFRT,CONTROL)
-#define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP)
+#define DEFIMP(CODE,NAME,GFRTDIRECT,GFRTF2C,GFRTGNU,CONTROL)
 #include "intrin.def"
 #undef DEFGEN
 #undef DEFSPEC
 #undef DEFIMP
-#undef DEFIMQ
 };
 
 
@@ -406,6 +398,10 @@ ffeintrin_check_ (ffeintrinImp imp, ffebldOp op,
 		    case 6:
 		      akt = 3;
 		      break;
+
+		    case 7:
+		      akt = ffecom_pointer_kind ();
+		      break;
 		    }
 		}
 	      okay &= anynum || (ffeinfo_kindtype (i) == akt);
@@ -527,11 +523,6 @@ ffeintrin_check_ (ffeintrinImp imp, ffebldOp op,
 	} while (TRUE);
     }
 
-  /* Ignore explicit trailing omitted args.  */
-
-  while ((arg != NULL) && (ffebld_head (arg) == NULL))
-    arg = ffebld_trail (arg);
-
   if (arg != NULL)
     return FFEBAD_INTRINSIC_TOOMANY;
 
@@ -601,6 +592,10 @@ ffeintrin_check_ (ffeintrinImp imp, ffebldOp op,
 	    case 6:
 	      kt = 3;
 	      break;
+
+	    case 7:
+	      kt = ffecom_pointer_kind ();
+	      break;
 	    }
 	}
       break;
@@ -611,10 +606,6 @@ ffeintrin_check_ (ffeintrinImp imp, ffebldOp op,
       kt = 1;
       break;
 
-    case 'p':
-      kt = ffecom_pointer_kind ();
-      break;
-
     case '=':
       need_col = TRUE;
       /* Fall through.  */
@@ -999,6 +990,10 @@ ffeintrin_check_ (ffeintrinImp imp, ffebldOp op,
 		    case 6:
 		      akt = 3;
 		      break;
+
+		    case 7:
+		      akt = ffecom_pointer_kind ();
+		      break;
 		    }
 		}
 	      okay &= anynum || (ffeinfo_kindtype (i) == akt);
@@ -1165,42 +1160,35 @@ ffeintrin_cmp_name_ (const void *name, const void *intrinsic)
   return ffesrc_strcmp_2c (ffe_case_intrin (), name, uc, lc, ic);
 }
 
-/* Return basic type of intrinsic implementation.  */
+/* Return basic type of intrinsic implementation, based on its
+   run-time implementation *only*.  (This is used only when
+   the type of an intrinsic name is needed without having a
+   list of arguments, i.e. an interface signature, such as when
+   passing the intrinsic itself, or really the run-time-library
+   function, as an argument.)
+
+   If there's no eligible intrinsic implementation, there must be
+   a bug somewhere else; no such reference should have been permitted
+   to go this far.  (Well, this might be wrong.)  */
 
 ffeinfoBasictype
 ffeintrin_basictype (ffeintrinSpec spec)
 {
+  ffeintrinImp imp;
+  ffecomGfrt gfrt;
+
   assert (spec < FFEINTRIN_spec);
-  return FFEINTRIN_specNONE;
-}
-
-/* Return code-generation implementation of intrinsic.
-
-   The idea is that an intrinsic might have its own implementation
-   (defined by the DEFIMP macro) or might defer to the implementation
-   of another intrinsic (defined by the DEFIMQ macro), and this is
-   what points to that other implementation.
-
-   The reason for this extra level of indirection, rather than
-   just adding "case" statements to the big switch in com.c's
-   ffecom_expr_intrinsic_ function, is so that generic disambiguation
-   can ensure that it doesn't have an ambiguity on its hands.
-   E.g. Both ABS and DABS might cope with a DOUBLE PRECISION,
-   etc.  Previously, the implementation itself was used to allow
-   multiple specific intrinsics to "accept" the argument list
-   if they all agreed on implementation.  But, since implementation
-   includes type signature and run-time-library function, another
-   level was needed to say "maybe two intrinsics would be handled
-   as two _different_ library references or involve different types
-   in general, but the specific code involved to implement them is
-   the same, so it is okay if a generic function reference can be
-   satisfied by either intrinsic".  */
-
-ffeintrinImp
-ffeintrin_codegen_imp (ffeintrinImp imp)
-{
+  imp = ffeintrin_specs_[spec].implementation;
   assert (imp < FFEINTRIN_imp);
-  return ffeintrin_imps_[imp].cg_imp;
+
+  if (ffe_is_f2c ())
+    gfrt = ffeintrin_imps_[imp].gfrt_f2c;
+  else 
+    gfrt = ffeintrin_imps_[imp].gfrt_gnu;
+
+  assert (gfrt != FFECOM_gfrt);
+
+  return ffecom_gfrt_basictype (gfrt);
 }
 
 /* Return family to which specific intrinsic belongs.  */
@@ -1240,7 +1228,6 @@ ffeintrin_fulfill_generic (ffebld *expr, ffeinfo *info, ffelexToken t)
   ffebad error;
   bool any = FALSE;
   bool highly_specific = FALSE;
-  char *name = NULL;
   int i;
 
   op = ffebld_op (*expr);
@@ -1268,13 +1255,10 @@ ffeintrin_fulfill_generic (ffebld *expr, ffeinfo *info, ffelexToken t)
       ffeIntrinsicState state
       = ffeintrin_state_family (ffeintrin_specs_[tspec].family);
       ffebad terror;
-      char *tname;
 
       if (state == FFE_intrinsicstateDELETED)
 	continue;
 
-      tname = ffeintrin_specs_[tspec].name;
-
       if (timp != FFEINTRIN_impNONE)
 	{
 	  if (!(ffeintrin_imps_[timp].control[0] == '-')
@@ -1295,33 +1279,13 @@ ffeintrin_fulfill_generic (ffebld *expr, ffeinfo *info, ffelexToken t)
 	    {
 	      if (imp != FFEINTRIN_impNONE)
 		{
-		  if (ffeintrin_imps_[timp].cg_imp
-		      == ffeintrin_imps_[imp].cg_imp)
-		    {
-		      if (ffebld_symter_specific (ffebld_left (*expr))
-			  == tspec)
-			{
-			  highly_specific = TRUE;
-			  imp = timp;
-			  spec = tspec;
-			  bt = tbt;
-			  kt = tkt;
-			  sz = tkt;
-			  error = terror;
-			}
-		      else if (nimp == FFEINTRIN_impNONE)
-			nimp = timp;
-		    }
-		  else
-		    {
-		      ffebad_start (FFEBAD_INTRINSIC_AMBIG);
-		      ffebad_here (0, ffelex_token_where_line (t),
-				   ffelex_token_where_column (t));
-		      ffebad_string (ffeintrin_gens_[gen].name);
-		      ffebad_string (ffeintrin_specs_[spec].name);
-		      ffebad_string (ffeintrin_specs_[tspec].name);
-		      ffebad_finish ();
-		    }
+		  ffebad_start (FFEBAD_INTRINSIC_AMBIG);
+		  ffebad_here (0, ffelex_token_where_line (t),
+			       ffelex_token_where_column (t));
+		  ffebad_string (ffeintrin_gens_[gen].name);
+		  ffebad_string (ffeintrin_specs_[spec].name);
+		  ffebad_string (ffeintrin_specs_[tspec].name);
+		  ffebad_finish ();
 		}
 	      else
 		{
@@ -1345,10 +1309,7 @@ ffeintrin_fulfill_generic (ffebld *expr, ffeinfo *info, ffelexToken t)
 	}
 
       if (error == FFEBAD)
-	{
-	  error = terror;
-	  name = tname;
-	}
+	error = terror;
     }
 
   if (any || (imp == FFEINTRIN_impNONE))
@@ -1357,12 +1318,10 @@ ffeintrin_fulfill_generic (ffebld *expr, ffeinfo *info, ffelexToken t)
 	{
 	  if (error == FFEBAD)
 	    error = FFEBAD_INTRINSIC_REF;
-	  if (name == NULL)
-	    name = ffeintrin_gens_[gen].name;
 	  ffebad_start (error);
 	  ffebad_here (0, ffelex_token_where_line (t),
 		       ffelex_token_where_column (t));
-	  ffebad_string (name);
+	  ffebad_string (ffeintrin_gens_[gen].name);
 	  ffebad_finish ();
 	}
 
@@ -1403,6 +1362,19 @@ ffeintrin_fulfill_generic (ffebld *expr, ffeinfo *info, ffelexToken t)
 				    : FFEINFO_kindFUNCTION,
 				    FFEINFO_whereINTRINSIC,
 				    sz));
+
+      if ((ffesymbol_attrs (ffebld_symter (symter)) & FFESYMBOL_attrsTYPE)
+	  && (((bt != ffesymbol_basictype (ffebld_symter (symter)))
+	       || (kt != ffesymbol_kindtype (ffebld_symter (symter)))
+	       || ((sz != FFETARGET_charactersizeNONE)
+		   && (sz != ffesymbol_size (ffebld_symter (symter)))))))
+	{
+	  ffebad_start (FFEBAD_INTRINSIC_TYPE);
+	  ffebad_here (0, ffelex_token_where_line (t),
+		       ffelex_token_where_column (t));
+	  ffebad_string (ffeintrin_gens_[gen].name);
+	  ffebad_finish ();
+	}
     }
 }
 
@@ -1428,6 +1400,7 @@ ffeintrin_fulfill_specific (ffebld *expr, ffeinfo *info,
 {
   ffebld symter;
   ffebldOp op;
+  ffeintrinGen gen;
   ffeintrinSpec spec;
   ffeintrinImp imp;
   ffeinfoBasictype bt = FFEINFO_basictypeNONE;
@@ -1436,14 +1409,21 @@ ffeintrin_fulfill_specific (ffebld *expr, ffeinfo *info,
   ffeIntrinsicState state;
   ffebad error;
   bool any = FALSE;
+  char *name;
 
   op = ffebld_op (*expr);
   assert ((op == FFEBLD_opFUNCREF) || (op == FFEBLD_opSUBRREF));
   assert (ffebld_op (ffebld_left (*expr)) == FFEBLD_opSYMTER);
 
+  gen = ffebld_symter_generic (ffebld_left (*expr));
   spec = ffebld_symter_specific (ffebld_left (*expr));
   assert (spec != FFEINTRIN_specNONE);
 
+  if (gen != FFEINTRIN_genNONE)
+    name = ffeintrin_gens_[gen].name;
+  else
+    name = ffeintrin_specs_[spec].name;
+
   state = ffeintrin_state_family (ffeintrin_specs_[spec].family);
 
   imp = ffeintrin_specs_[spec].implementation;
@@ -1469,15 +1449,10 @@ ffeintrin_fulfill_specific (ffebld *expr, ffeinfo *info,
     {
       if (!any)
 	{
-	  char *name;
 
 	  ffebad_start (error);
 	  ffebad_here (0, ffelex_token_where_line (t),
 		       ffelex_token_where_column (t));
-	  if (spec != FFEINTRIN_specNONE)
-	    name = ffeintrin_specs_[spec].name;
-	  else
-	    name = ffeintrin_imps_[imp].name;
 	  ffebad_string (name);
 	  ffebad_finish ();
 	}
@@ -1503,20 +1478,47 @@ ffeintrin_fulfill_specific (ffebld *expr, ffeinfo *info,
 				    : FFEINFO_kindFUNCTION,
 				    FFEINFO_whereINTRINSIC,
 				    sz));
+
+      if ((ffesymbol_attrs (ffebld_symter (symter)) & FFESYMBOL_attrsTYPE)
+	  && (((bt != ffesymbol_basictype (ffebld_symter (symter)))
+	       || (kt != ffesymbol_kindtype (ffebld_symter (symter)))
+	       || (sz != ffesymbol_size (ffebld_symter (symter))))))
+	{
+	  ffebad_start (FFEBAD_INTRINSIC_TYPE);
+	  ffebad_here (0, ffelex_token_where_line (t),
+		       ffelex_token_where_column (t));
+	  ffebad_string (name);
+	  ffebad_finish ();
+	}
     }
 }
 
-/* Return run-time index of intrinsic implementation as arg.  */
+/* Return run-time index of intrinsic implementation as direct call.  */
 
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
 ffecomGfrt
-ffeintrin_gfrt (ffeintrinImp imp)
+ffeintrin_gfrt_direct (ffeintrinImp imp)
 {
   assert (imp < FFEINTRIN_imp);
-  return ffeintrin_imps_[imp].gfrt;
-}
 
+  return ffeintrin_imps_[imp].gfrt_direct;
+}
 #endif
+
+/* Return run-time index of intrinsic implementation as actual argument.  */
+
+#if FFECOM_targetCURRENT == FFECOM_targetGCC
+ffecomGfrt
+ffeintrin_gfrt_indirect (ffeintrinImp imp)
+{
+  assert (imp < FFEINTRIN_imp);
+
+  if (! ffe_is_f2c ())
+    return ffeintrin_imps_[imp].gfrt_gnu;
+  return ffeintrin_imps_[imp].gfrt_f2c;
+}
+#endif
+
 void
 ffeintrin_init_0 ()
 {
@@ -1571,14 +1573,14 @@ ffeintrin_init_0 ()
 
       if ((c[0] != '-')
 	  && (c[0] != 'A')
-      && (c[0] != 'C')
-      && (c[0] != 'I')
-      && (c[0] != 'L')
-      && (c[0] != 'R')
-      && (c[0] != 'B')
-      && (c[0] != 'F')
-      && (c[0] != 'N')
-      && (c[0] != 'S'))
+	  && (c[0] != 'C')
+	  && (c[0] != 'I')
+	  && (c[0] != 'L')
+	  && (c[0] != 'R')
+	  && (c[0] != 'B')
+	  && (c[0] != 'F')
+	  && (c[0] != 'N')
+	  && (c[0] != 'S'))
 	{
 	  fprintf (stderr, "%s: bad return-base-type\n",
 		   ffeintrin_imps_[i].name);
@@ -1586,10 +1588,9 @@ ffeintrin_init_0 ()
 	}
       if ((c[1] != '-')
 	  && (c[1] != '=')
-      && ((c[1] < '1')
-	  || (c[1] > '9'))
-	  && (c[1] != 'C')
-	  && (c[1] != 'p'))
+	  && ((c[1] < '1')
+	      || (c[1] > '9'))
+	  && (c[1] != 'C'))
 	{
 	  fprintf (stderr, "%s: bad return-kind-type\n",
 		   ffeintrin_imps_[i].name);
@@ -1615,8 +1616,8 @@ ffeintrin_init_0 ()
 	}
       if ((c[colon + 1] != '-')
 	  && (c[colon + 1] != '*')
-      && ((c[colon + 1] < '0')
-	  || (c[colon + 1] > '9')))
+	  && ((c[colon + 1] < '0')
+	      || (c[colon + 1] > '9')))
 	{
 	  fprintf (stderr, "%s: bad COL-spec\n",
 		   ffeintrin_imps_[i].name);
@@ -1627,7 +1628,7 @@ ffeintrin_init_0 ()
 	{
 	  while ((c[0] != '=')
 		 && (c[0] != ',')
-	  && (c[0] != '\0'))
+		 && (c[0] != '\0'))
 	    ++c;
 	  if (c[0] != '=')
 	    {
@@ -1637,28 +1638,27 @@ ffeintrin_init_0 ()
 	    }
 	  if ((c[1] == '?')
 	      || (c[1] == '!')
-	  || (c[1] == '!')
 	      || (c[1] == '+')
-	  || (c[1] == '*')
+	      || (c[1] == '*')
 	      || (c[1] == 'n')
-	  || (c[1] == 'p'))
+	      || (c[1] == 'p'))
 	    ++c;
 	  if (((c[1] != '-')
 	       && (c[1] != 'A')
-	  && (c[1] != 'C')
-	  && (c[1] != 'I')
-	  && (c[1] != 'L')
-	  && (c[1] != 'R')
-	  && (c[1] != 'B')
-	  && (c[1] != 'F')
-	  && (c[1] != 'N')
-	  && (c[1] != 'S')
-	  && (c[1] != 'g')
-	  && (c[1] != 's'))
+	       && (c[1] != 'C')
+	       && (c[1] != 'I')
+	       && (c[1] != 'L')
+	       && (c[1] != 'R')
+	       && (c[1] != 'B')
+	       && (c[1] != 'F')
+	       && (c[1] != 'N')
+	       && (c[1] != 'S')
+	       && (c[1] != 'g')
+	       && (c[1] != 's'))
 	      || ((c[2] != '*')
 		  && ((c[2] < '1')
 		      || (c[2] > '9'))
-	      && (c[2] != 'A')))
+		  && (c[2] != 'A')))
 	    {
 	      fprintf (stderr, "%s: bad arg-type\n",
 		       ffeintrin_imps_[i].name);
@@ -1695,13 +1695,13 @@ ffeintrin_init_0 ()
 	    ++c;
 	  if ((c[3] == '&')
 	      || (c[3] == 'i')
-	  || (c[3] == 'w')
-	  || (c[3] == 'x'))
+	      || (c[3] == 'w')
+	      || (c[3] == 'x'))
 	    ++c;
 	  if (c[3] == ',')
 	    {
 	      c += 4;
-	      break;
+	      continue;
 	    }
 	  if (c[3] != '\0')
 	    {
@@ -1713,7 +1713,7 @@ ffeintrin_init_0 ()
     }
 }
 
-/* Determine whether intrinsic ok as actual arg.  */
+/* Determine whether intrinsic is okay as an actual argument.  */
 
 bool
 ffeintrin_is_actualarg (ffeintrinSpec spec)
@@ -1727,8 +1727,11 @@ ffeintrin_is_actualarg (ffeintrinSpec spec)
 
   return (!ffe_is_pedantic () || ffeintrin_specs_[spec].is_actualarg)
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
-    && (ffeintrin_imps_[ffeintrin_specs_[spec].implementation].gfrt
-	!= FFECOM_gfrt)
+    && (ffe_is_f2c ()
+	? (ffeintrin_imps_[ffeintrin_specs_[spec].implementation].gfrt_f2c
+	   != FFECOM_gfrt)
+	: (ffeintrin_imps_[ffeintrin_specs_[spec].implementation].gfrt_gnu
+	   != FFECOM_gfrt))
 #endif
     && ((state == FFE_intrinsicstateENABLED)
 	|| (state == FFE_intrinsicstateHIDDEN));
@@ -1822,7 +1825,10 @@ ffeintrin_is_intrinsic (char *name, ffelexToken t, bool explicit,
 
   if (spec != FFEINTRIN_specNONE)
     {
-      name = ffeintrin_specs_[spec].name;
+      if (gen != FFEINTRIN_genNONE)
+	name = ffeintrin_gens_[gen].name;
+      else
+	name = ffeintrin_specs_[spec].name;
 
       if (((state = ffeintrin_state_family (ffeintrin_specs_[spec].family))
 	   == FFE_intrinsicstateDELETED)
@@ -1907,13 +1913,51 @@ ffeintrin_is_intrinsic (char *name, ffelexToken t, bool explicit,
   return TRUE;
 }
 
-/* Return kind type of intrinsic implementation.  */
+/* Return TRUE if intrinsic is standard F77 (or, if -ff90, F90).  */
+
+bool
+ffeintrin_is_standard (ffeintrinGen gen, ffeintrinSpec spec)
+{
+  if (spec == FFEINTRIN_specNONE)
+    {
+      if (gen == FFEINTRIN_genNONE)
+	return FALSE;
+
+      spec = ffeintrin_gens_[gen].specs[0];
+      if (spec == FFEINTRIN_specNONE)
+	return FALSE;
+    }
+
+  if ((ffeintrin_specs_[spec].family == FFEINTRIN_familyF77)
+      || (ffe_is_90 ()
+	  && ((ffeintrin_specs_[spec].family == FFEINTRIN_familyF90)
+	      || (ffeintrin_specs_[spec].family == FFEINTRIN_familyMIL)
+	      || (ffeintrin_specs_[spec].family == FFEINTRIN_familyASC))))
+    return TRUE;
+  return FALSE;
+}
+
+/* Return kind type of intrinsic implementation.  See ffeintrin_basictype,
+   its sibling.  */
 
 ffeinfoKindtype
 ffeintrin_kindtype (ffeintrinSpec spec)
 {
+  ffeintrinImp imp;
+  ffecomGfrt gfrt;
+
   assert (spec < FFEINTRIN_spec);
-  return FFEINFO_kindtypeNONE;
+  imp = ffeintrin_specs_[spec].implementation;
+  assert (imp < FFEINTRIN_imp);
+
+  if (ffe_is_f2c ())
+    gfrt = ffeintrin_imps_[imp].gfrt_f2c;
+  else
+    gfrt = ffeintrin_imps_[imp].gfrt_gnu;
+
+  assert (gfrt != FFECOM_gfrt);
+
+  return ffecom_gfrt_kindtype (gfrt);
 }
 
 /* Return name of generic intrinsic.  */
@@ -1994,6 +2038,10 @@ ffeintrin_state_family (ffeintrinFamily family)
       state = ffe_intrinsic_state_unix ();
       return state;
 
+    case FFEINTRIN_familyBADU77:
+      state = ffe_intrinsic_state_badu77 ();
+      return state;
+
     default:
       assert ("bad family" == NULL);
       return FFE_intrinsicstateDELETED;
