@@ -103,6 +103,10 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   "-fcase-upper",
   "-fcase-lower",
   "-fcase-preserve",
+  "-fbadu77-intrinsics-delete",
+  "-fbadu77-intrinsics-hide",
+  "-fbadu77-intrinsics-disable",
+  "-fbadu77-intrinsics-enable",
   "-fdcp-intrinsics-delete",
   "-fdcp-intrinsics-hide",
   "-fdcp-intrinsics-disable",
@@ -115,6 +119,10 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   "-ff90-intrinsics-hide",
   "-ff90-intrinsics-disable",
   "-ff90-intrinsics-enable",
+  "-fgnu-intrinsics-delete",
+  "-fgnu-intrinsics-hide",
+  "-fgnu-intrinsics-disable",
+  "-fgnu-intrinsics-enable",
   "-fmil-intrinsics-delete",
   "-fmil-intrinsics-hide",
   "-fmil-intrinsics-disable",
