@@ -91,6 +91,7 @@ extern bool ffe_is_f2c_;
 extern bool ffe_is_f2c_library_;
 extern bool ffe_is_ffedebug_;
 extern bool ffe_is_free_form_;
+extern bool ffe_is_globals_;
 extern bool ffe_is_ident_;
 extern bool ffe_is_init_local_zero_;
 extern bool ffe_is_mainprog_;
@@ -108,6 +109,7 @@ extern bool ffe_is_ugly_init_;
 extern bool ffe_is_ugly_logint_;
 extern bool ffe_is_version_;
 extern bool ffe_is_vxt_;
+extern bool ffe_is_warn_globals_;
 extern bool ffe_is_warn_implicit_;
 extern bool ffe_is_warn_surprising_;
 extern bool ffe_is_zeros_;
@@ -115,6 +117,7 @@ extern ffeCase ffe_case_intrin_;
 extern ffeCase ffe_case_match_;
 extern ffeCase ffe_case_source_;
 extern ffeCase ffe_case_symbol_;
+extern ffeIntrinsicState ffe_intrinsic_state_badu77_;
 extern ffeIntrinsicState ffe_intrinsic_state_gnu_;
 extern ffeIntrinsicState ffe_intrinsic_state_f2c_;
 extern ffeIntrinsicState ffe_intrinsic_state_f90_;
@@ -157,6 +160,7 @@ void ffe_terminate_4 (void);
 #define ffe_case_match() ffe_case_match_
 #define ffe_case_source() ffe_case_source_
 #define ffe_case_symbol() ffe_case_symbol_
+#define ffe_intrinsic_state_badu77() ffe_intrinsic_state_badu77_
 #define ffe_intrinsic_state_f2c() ffe_intrinsic_state_f2c_
 #define ffe_intrinsic_state_f90() ffe_intrinsic_state_f90_
 #define ffe_intrinsic_state_gnu() ffe_intrinsic_state_gnu_
@@ -174,6 +178,7 @@ void ffe_terminate_4 (void);
 #define ffe_is_f2c_library() ffe_is_f2c_library_
 #define ffe_is_ffedebug() ffe_is_ffedebug_
 #define ffe_is_free_form() ffe_is_free_form_
+#define ffe_is_globals() ffe_is_globals_
 #define ffe_is_ident() ffe_is_ident_
 #define ffe_is_init_local_zero() ffe_is_init_local_zero_
 #define ffe_is_mainprog() ffe_is_mainprog_
@@ -194,6 +199,7 @@ void ffe_terminate_4 (void);
 #define ffe_is_underscoring() ffe_is_underscoring_
 #define ffe_is_version() ffe_is_version_
 #define ffe_is_vxt() ffe_is_vxt_
+#define ffe_is_warn_globals() ffe_is_warn_globals_
 #define ffe_is_warn_implicit() ffe_is_warn_implicit_
 #define ffe_is_warn_surprising() ffe_is_warn_surprising_
 #define ffe_is_zeros() ffe_is_zeros_
@@ -205,6 +211,7 @@ void ffe_terminate_4 (void);
 #define ffe_set_case_match(f) (ffe_case_match_ = (f))
 #define ffe_set_case_source(f) (ffe_case_source_ = (f))
 #define ffe_set_case_symbol(f) (ffe_case_symbol_ = (f))
+#define ffe_set_intrinsic_state_badu77(s) (ffe_intrinsic_state_badu77_ = (s))
 #define ffe_set_intrinsic_state_f2c(s) (ffe_intrinsic_state_f2c_ = (s))
 #define ffe_set_intrinsic_state_f90(s) (ffe_intrinsic_state_f90_ = (s))
 #define ffe_set_intrinsic_state_gnu(s) (ffe_intrinsic_state_gnu_ = (s))
@@ -222,6 +229,7 @@ void ffe_terminate_4 (void);
 #define ffe_set_is_f2c_library(f) (ffe_is_f2c_library_ = (f))
 #define ffe_set_is_ffedebug(f) (ffe_is_ffedebug_ = (f))
 #define ffe_set_is_free_form(f) (ffe_is_free_form_ = (f))
+#define ffe_set_is_globals(f) (ffe_is_globals_ = (f))
 #define ffe_set_is_ident(f) (ffe_is_ident_ = (f))
 #define ffe_set_is_init_local_zero(f) (ffe_is_init_local_zero_ = (f))
 #define ffe_set_is_mainprog(f) (ffe_is_mainprog_ = (f))
@@ -241,6 +249,7 @@ void ffe_terminate_4 (void);
 #define ffe_set_is_underscoring(f) (ffe_is_underscoring_ = (f))
 #define ffe_set_is_version(f) (ffe_is_version_ = (f))
 #define ffe_set_is_vxt(f) (ffe_is_vxt_ = (f))
+#define ffe_set_is_warn_globals(f) (ffe_is_warn_globals_ = (f))
 #define ffe_set_is_warn_implicit(f) (ffe_is_warn_implicit_ = (f))
 #define ffe_set_is_warn_surprising(f) (ffe_is_warn_surprising_ = (f))
 #define ffe_set_is_zeros(f) (ffe_is_zeros_ = (f))
